@@ -1,6 +1,6 @@
 /*
  *  smpeg-xmms - (S)MPEG Plugin for XMMS
- *  Copyright (C) 2000 4Front Technologies
+ *  Copyright (C) 2000-2001 4Front Technologies
  *  http://www.opensound.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -16,7 +16,7 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
+ */ 
 
 #include <gtk/gtk.h>
 #include <smpeg.h>
@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <config.h>
 #include "smpeg-xmms.h"
 #include <xmms/plugin.h>
 #include <xmms/xmmsctrl.h>
@@ -48,7 +49,7 @@ static InputPlugin smpeg_ip;
 
 SmpegConfig smpeg_cfg;
 
-static SDL_Thread *thread, *vcd_thread;
+static SDL_Thread *thread, *vcd_thread, *audio_thread;
 static SDL_mutex *mpeg_mutex;
 static SMPEG *mpeg = NULL;
 static SDL_Surface *screen = NULL;
@@ -56,13 +57,16 @@ static gboolean playing = FALSE;
 static gboolean doublesize = FALSE;
 static gboolean paused = FALSE;
 static gboolean fullscreen = FALSE;
+static gboolean maximised = FALSE;
 static gboolean is_vcd = FALSE;
 static gboolean is_stream = FALSE;
 static gboolean rstop = FALSE;
 static gboolean bilinear_filtering = FALSE;
+static gboolean XMMSAUDIO = FALSE;
+static gboolean display = FALSE;
 static int streamfd;
 
-static gint vol;
+static gint vol, nch, srate, bitrate;
 static gint size_w, size_h;
 
 #ifdef __linux
@@ -83,10 +87,13 @@ static void smpeg_get_song_info(gchar * filename, gchar ** title,
 				gint * length);
 static void smpeg_about(void);
 static void smpeg_playback_func(void);
+static void smpeg_audio_func(void *arg);
 
 static void smpeg_set_doublesize(gboolean ds);
 static void smpeg_set_fullscreen(gboolean fs);
 
+static SDL_Rect maximise_dimensions;
+
 gint ctrlsocket_get_session_id(void);	/* FIXME */
 
 InputPlugin *get_iplugin_info(void)
@@ -103,6 +110,8 @@ InputPlugin *get_iplugin_info(void)
 	smpeg_ip.get_song_info = smpeg_get_song_info;
 	smpeg_ip.about = smpeg_about;
 	smpeg_ip.configure = smpeg_configure;
+	smpeg_ip.add_vis_pcm = NULL;
+	smpeg_ip.output = NULL;
 	return &smpeg_ip;
 }
 
@@ -114,7 +123,13 @@ static gint smpeg_is_our_file(gchar * filename)
 	if (!strncasecmp(filename, "vcd:", 4) ||
 	    (ext && (!strcasecmp(ext, ".mpg") || !strcasecmp(ext, ".mpeg") ||
 	    !strcasecmp(ext, ".bin"))))
+	{
+		FILE *g = fopen (filename, "r");
+		if (!g)
+			return FALSE;
+		fclose(g);
 		return TRUE;
+	}
 	return FALSE;
 }
 
@@ -203,10 +218,10 @@ static void smpeg_playback_func(void)
 							}
 							break;
 						case SDLK_PAGEUP:
-							xmms_remote_playlist_next(ctrlsocket_get_session_id());
+							xmms_remote_playlist_prev(ctrlsocket_get_session_id());
 							break;
 						case SDLK_PAGEDOWN:
-							xmms_remote_playlist_prev(ctrlsocket_get_session_id());
+							xmms_remote_playlist_next(ctrlsocket_get_session_id());
 							break;
 						case SDLK_RETURN:
 							smpeg_set_fullscreen(!fullscreen);
@@ -221,9 +236,9 @@ static void smpeg_playback_func(void)
 							xmms_remote_set_main_volume(ctrlsocket_get_session_id(), vol);
 							break;
 						case SDLK_p:
-							if(fullscreen)
-								smpeg_set_fullscreen(!fullscreen);
 							if ( event.key.keysym.mod & KMOD_CTRL ) {
+								if(fullscreen)
+									smpeg_set_fullscreen(!fullscreen);
 								GDK_THREADS_ENTER();
 								smpeg_configure();
 								GDK_THREADS_LEAVE();
@@ -360,6 +375,19 @@ static void smpeg_playback_func(void)
 						case 3:
 							smpeg_set_fullscreen(!fullscreen);
 							break;
+						case 4:
+							i = xmms_remote_get_main_volume(ctrlsocket_get_session_id());
+							i += 5;
+							if (i > 100)
+								i = 100;
+							xmms_remote_set_main_volume(ctrlsocket_get_session_id(), i);
+							break;
+						case 5:
+							i = xmms_remote_get_main_volume(ctrlsocket_get_session_id());
+							i -= 5;
+							if (i < 0)
+								i = 0;
+							xmms_remote_set_main_volume(ctrlsocket_get_session_id(), i);
 					}
 					break;
 				case SDL_QUIT: {
@@ -428,14 +456,13 @@ static void smpeg_set_doublesize(gboolean ds)
 	SDL_mutexP(mpeg_mutex);
 	SMPEG_getinfo(mpeg, &info);
 
-	smpeg_set_fullscreen(FALSE);
-			
+	if (fullscreen)
+		smpeg_set_fullscreen(!fullscreen);
 	if (SMPEG_status(mpeg) == SMPEG_PLAYING)
 	{
 		SMPEG_pause(mpeg);
 		unpause = TRUE;
 	}
-
 	if (info.has_video)
 	{
 		if (ds == TRUE)
@@ -454,7 +481,9 @@ static void smpeg_set_doublesize(gboolean ds)
 		SMPEG_double(mpeg, ds);
 		smpeg_set_caption(NULL, info.width, info.height, ds);
 		if (unpause)
+		{
 			SMPEG_pause(mpeg);
+		}
 	}
 	doublesize = ds;
 	SDL_mutexV(mpeg_mutex);
@@ -499,15 +528,22 @@ static void smpeg_set_fullscreen(gboolean fs)
 			{
 				for (num_modes = 0; modes[num_modes];
 				     num_modes++) ;
-
-				for (i = num_modes - 1; i >= 0; i--)
+				if (smpeg_cfg.samevideo_mode)
 				{
-					if ((modes[i]->w > info.width)
-					    && (modes[i]->h > info.height))
+					w = modes[0]->w;
+					h = modes[0]->h;
+				}
+				else
+				{
+					for (i = num_modes - 1; i >= 0; i--)
 					{
-						w = modes[i]->w;
-						h = modes[i]->h;
-						break;
+						if ((modes[i]->w > info.width)
+						    && (modes[i]->h > info.height))
+						{
+							w = modes[i]->w;
+							h = modes[i]->h;
+							break;
+						}
 					}
 				} 
 				if (w != 0 && h != 0)
@@ -543,6 +579,7 @@ static void smpeg_set_fullscreen(gboolean fs)
 	SDL_mutexV(mpeg_mutex);
 }
 
+
 #ifdef __linux
 
 static void set_track_number(int track, char *m, char *s, char *f)
@@ -618,13 +655,14 @@ static void smpeg_read_vcd_func(void)
 static void smpeg_play_file(gchar * filename)
 {
 	SMPEG_Info info;
-	gint bitrate = 0, srate = 0, nch = 1;
 	gchar *name, *error;
 	Uint32 video_flags;
 	struct stat buf;	
 	gchar *pip;
 	gint track;
 	char m, s, f;
+	smpeg_read_config();
+	XMMSAUDIO = smpeg_cfg.xmmsaudio_mode;
 
 	if(!strncasecmp(filename, "vcd:", 4))
 	{
@@ -669,7 +707,10 @@ static void smpeg_play_file(gchar * filename)
 		}
 		vcd_mutex = SDL_CreateMutex();
 		vcd_thread = SDL_CreateThread((void *)smpeg_read_vcd_func, NULL);
-		mpeg = SMPEG_new_descr(vcd_pipe[0], &info, TRUE);
+		if(XMMSAUDIO)
+			mpeg = SMPEG_new_descr(vcd_pipe[0], &info, FALSE);
+		else
+			mpeg = SMPEG_new_descr(vcd_pipe[0], &info, TRUE);
 #endif
 	}
 	else
@@ -678,13 +719,19 @@ static void smpeg_play_file(gchar * filename)
 		if((stat(filename, &buf) < 0) || !S_ISFIFO(buf.st_mode))
 		{
 			is_stream = FALSE;
-			mpeg = SMPEG_new(filename, &info, TRUE);
+			if(XMMSAUDIO)
+				mpeg = SMPEG_new(filename, &info, FALSE);
+			else
+				mpeg = SMPEG_new(filename, &info, TRUE);
 		}
 		else
 		{
 			streamfd = open(filename, O_RDONLY, 0);
 			is_stream = TRUE;
-			mpeg = SMPEG_new_descr(streamfd, &info, TRUE);
+			if(XMMSAUDIO)
+				mpeg = SMPEG_new_descr(streamfd, &info, FALSE);
+			else
+				mpeg = SMPEG_new_descr(streamfd, &info, TRUE);
 		}
 	}
 	if ((error = SMPEG_error(mpeg)) != NULL)
@@ -697,13 +744,14 @@ static void smpeg_play_file(gchar * filename)
 	if (!is_vcd && access(filename, R_OK) != 0)
 		g_error("smpeg_play_file(): File not readable, but SMPEG reported no error");
 
-
-	SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO);
-	video_flags = SDL_SWSURFACE | SDL_ASYNCBLIT | SDL_RESIZABLE;
+	if (!smpeg_cfg.audioonly_mode)
+	{
+		SDL_Init(SDL_INIT_VIDEO);
+		video_flags = SDL_SWSURFACE | SDL_ASYNCBLIT | SDL_RESIZABLE;
+	}
 	mpeg_mutex = SDL_CreateMutex();
 
 	SDL_mutexP(mpeg_mutex);
-	smpeg_read_config();
 
 	if (smpeg_cfg.center_mode)
 		putenv("SDL_VIDEO_CENTERED=1");
@@ -714,41 +762,47 @@ static void smpeg_play_file(gchar * filename)
 		putenv("SDL_VIDEO_CENTERED=0");
 #endif
 
-	if (smpeg_cfg.states_mode && smpeg_cfg.windowsize_mode) {
-		gint wg = 0, hg = 0;
-		SMPEG_getinfo(mpeg, &info);
-		wg = ((smpeg_cfg.sizeh_mode / (gdouble)info.height) * info.width);
-		hg = ((smpeg_cfg.sizew_mode / (gdouble)info.width) * info.height);
-		screen = SDL_SetVideoMode(smpeg_cfg.sizew_mode, smpeg_cfg.sizeh_mode, 0,
-					   video_flags);
-		if (!smpeg_cfg.wratio_mode) {
-			if (wg < smpeg_cfg.sizew_mode)
-				SMPEG_scaleXY(mpeg, wg, smpeg_cfg.sizeh_mode);
-			else
-				SMPEG_scaleXY(mpeg, smpeg_cfg.sizew_mode, hg);
-		}
-		else if (smpeg_cfg.wratio_mode)
-			SMPEG_scaleXY(mpeg, smpeg_cfg.sizew_mode, smpeg_cfg.sizeh_mode);
-	}
-	else {
-		if(smpeg_cfg.double_mode) {
-			screen = SDL_SetVideoMode(info.width * 2, info.height * 2, 0,
-						  video_flags);
-			SMPEG_double(mpeg, smpeg_cfg.double_mode);
+	if(!smpeg_cfg.audioonly_mode)
+	{
+		if (smpeg_cfg.states_mode && smpeg_cfg.windowsize_mode) {
+			gint wg = 0, hg = 0;
+			SMPEG_getinfo(mpeg, &info);
+			wg = ((smpeg_cfg.sizeh_mode / (gdouble)info.height) * info.width);
+			hg = ((smpeg_cfg.sizew_mode / (gdouble)info.width) * info.height);
+			screen = SDL_SetVideoMode(smpeg_cfg.sizew_mode, smpeg_cfg.sizeh_mode, 0,
+						   video_flags);
+			if (!smpeg_cfg.wratio_mode) {
+				if (wg < smpeg_cfg.sizew_mode)
+					SMPEG_scaleXY(mpeg, wg, smpeg_cfg.sizeh_mode);
+				else
+					SMPEG_scaleXY(mpeg, smpeg_cfg.sizew_mode, hg);
+			}
+			else if (smpeg_cfg.wratio_mode)
+				SMPEG_scaleXY(mpeg, smpeg_cfg.sizew_mode, smpeg_cfg.sizeh_mode);
 		}
 		else {
-			screen = SDL_SetVideoMode(info.width, info.height, 0,
-						  video_flags);
-			SMPEG_double(mpeg, smpeg_cfg.double_mode);
+			if(smpeg_cfg.double_mode) {
+				screen = SDL_SetVideoMode(info.width * 2, info.height * 2, 0,
+							  video_flags);
+				SMPEG_double(mpeg, smpeg_cfg.double_mode);
+			}
+			else {
+				screen = SDL_SetVideoMode(info.width, info.height, 0,
+							  video_flags);
+				SMPEG_double(mpeg, smpeg_cfg.double_mode);
+			}
 		}
+		display = TRUE;
 	}
-
-	SMPEG_setdisplay(mpeg, screen, NULL, NULL);
-	SMPEG_enablevideo(mpeg, TRUE);
-	SMPEG_enableaudio(mpeg, TRUE);
+	if(!smpeg_cfg.audioonly_mode)
+		SMPEG_setdisplay(mpeg, screen, NULL, NULL);
+	if(smpeg_cfg.audioonly_mode)
+		SMPEG_enablevideo(mpeg, FALSE);
+	else
+		SMPEG_enablevideo(mpeg, TRUE);
 	SMPEG_loop(mpeg, FALSE);
-/*	Note: seems to do strange things sometime.
-	SDL_EnableKeyRepeat(300, 50); */
+	if(XMMSAUDIO)
+		smpeg_ip.output->open_audio(FMT_S16_LE, 44100, 2);
 	SMPEG_play(mpeg);
 	if (smpeg_cfg.fullscreen_mode)
 		smpeg_set_fullscreen(!fullscreen);
@@ -757,7 +811,7 @@ static void smpeg_play_file(gchar * filename)
 		SMPEG_Filter *filter = SMPEGfilter_bilinear();
 		filter = SMPEG_filter( mpeg, filter );
 		filter->destroy(filter);
-	}		
+	}
 	SDL_mutexV(mpeg_mutex);
 
 	if(is_vcd)
@@ -817,11 +871,48 @@ static void smpeg_play_file(gchar * filename)
 #endif
 	playing = TRUE;
 	bilinear_filtering = smpeg_cfg.filter_mode;
+	if(!smpeg_cfg.audioonly_mode)
+		maximise_dimensions = *SDL_ListModes(NULL, SDL_SWSURFACE | SDL_ASYNCBLIT | SDL_FULLSCREEN)[0];
 	fullscreen = smpeg_cfg.fullscreen_mode;
 	doublesize = smpeg_cfg.double_mode;
+	if(XMMSAUDIO)
+		audio_thread = SDL_CreateThread((void *) smpeg_audio_func, NULL);
 	thread = SDL_CreateThread((void *) smpeg_playback_func, NULL);
 }
 
+static void smpeg_audio_func(void *arg)
+{
+	char data[512*4];
+	gint bytes, blk_size;
+	SMPEG_Info info;
+	SDL_AudioSpec audiofmt;
+	audiofmt.format=AUDIO_S16LSB;
+	audiofmt.freq=srate;
+	audiofmt.size=2048;
+	audiofmt.channels=nch;
+	SDL_mutexP(mpeg_mutex);
+	SMPEG_actualSpec(mpeg, &audiofmt);
+	SDL_mutexV(mpeg_mutex);
+	bytes = 512*2*nch;
+	while(playing && !paused)
+	{
+		SDL_mutexP(mpeg_mutex);
+		SMPEG_playAudio(mpeg, data, bytes);
+		SDL_mutexV(mpeg_mutex);
+		while(smpeg_ip.output->buffer_free() < bytes && playing)
+		{
+			xmms_usleep(10000);
+		}
+		if(SMPEG_status(mpeg) == SMPEG_PLAYING && !paused)
+		{
+			smpeg_ip.output->write_audio(data, bytes);
+			smpeg_ip.add_vis_pcm(smpeg_ip.output->written_time(), FMT_S16_LE, nch, bytes, data);
+		}
+		memset(data, 0, bytes);
+	}
+	SDL_mutexV(mpeg_mutex);
+}
+
 static gint smpeg_timeout_func(gpointer data)
 {
 	if(xmms_remote_is_playing(0)) {
@@ -841,7 +932,6 @@ static gint smpeg_timeout_func(gpointer data)
 	}
 	else {
 		SDL_Quit();
-		// exit(0);
 		return FALSE;
 	}
 }
@@ -851,26 +941,34 @@ static void smpeg_stop(void)
 	gint timeout_tag = 0;
 	if (!playing)
 		return;
-	if (smpeg_cfg.states_mode)
+	if(!smpeg_cfg.audioonly_mode || display)
 	{
-		ConfigFile *cfg = xmms_cfg_open_default_file();
-		gint size_w, size_h;
-		size_w = screen->w;
-		size_h = screen->h;
-		xmms_cfg_write_boolean(cfg, "smpeg-xmms", "smpeg_double",
-				       doublesize);
-		xmms_cfg_write_boolean(cfg, "smpeg-xmms", "smpeg_fullscreen",
-				       fullscreen);
-		if(!fullscreen && smpeg_cfg.windowsize_mode) {
-		xmms_cfg_write_int(cfg, "smpeg-xmms", "smpeg_sizew",
-				   size_w);
-		xmms_cfg_write_int(cfg, "smpeg-xmms", "smpeg_sizeh",
-				   size_h);
+		if (smpeg_cfg.states_mode)
+		{
+			ConfigFile *cfg = xmms_cfg_open_default_file();
+			gint size_w, size_h;
+			size_w = screen->w;
+			size_h = screen->h;
+			xmms_cfg_write_boolean(cfg, "smpeg-xmms", "smpeg_double",
+					       doublesize);
+			xmms_cfg_write_boolean(cfg, "smpeg-xmms", "smpeg_fullscreen",
+					       fullscreen);
+			if(!fullscreen && smpeg_cfg.windowsize_mode) {
+			xmms_cfg_write_int(cfg, "smpeg-xmms", "smpeg_sizew",
+					   size_w);
+			xmms_cfg_write_int(cfg, "smpeg-xmms", "smpeg_sizeh",
+					   size_h);
+			}
+			xmms_cfg_write_default_file(cfg);
+			xmms_cfg_free(cfg);
 		}
-		xmms_cfg_write_default_file(cfg);
-		xmms_cfg_free(cfg);
 	}
 	SDL_mutexP(mpeg_mutex);
+	if(XMMSAUDIO)
+	{
+		SDL_KillThread(audio_thread);
+		smpeg_ip.output->close_audio();
+	}
 	SDL_KillThread(thread);
 	SMPEG_stop(mpeg);
 	SMPEG_delete(mpeg);
@@ -888,13 +986,19 @@ static void smpeg_stop(void)
 #endif
 	SDL_mutexV(mpeg_mutex);
 	SDL_DestroyMutex(mpeg_mutex);
-	SDL_FreeSurface(screen);
-	if(!smpeg_cfg.window_mode && !rstop)
-		timeout_tag = gtk_timeout_add(0, smpeg_timeout_func, NULL);
-	if(smpeg_cfg.window_mode || fullscreen)
-		SDL_Quit();
-	if(!smpeg_cfg.window_mode && rstop)
-		SDL_Quit();
+	if(!smpeg_cfg.audioonly_mode || display)
+	{
+		SDL_FreeSurface(screen);
+		if (smpeg_cfg.audioonly_mode)
+			SDL_Quit();
+		if(!smpeg_cfg.window_mode && !rstop)
+			timeout_tag = gtk_timeout_add(0, smpeg_timeout_func, NULL);
+		if(smpeg_cfg.window_mode || fullscreen)
+			SDL_Quit();
+		if(!smpeg_cfg.window_mode && rstop)
+			SDL_Quit();
+		display = FALSE;
+	}
 	playing = FALSE;
 	fullscreen = FALSE;
 	paused = FALSE;
@@ -912,6 +1016,10 @@ static void smpeg_pause(short p)
 	if (p == paused)
 		g_error("Pause confusion");
 	SMPEG_pause(mpeg);
+	if(XMMSAUDIO)
+	{
+		smpeg_ip.output->pause(p);
+	}
 	paused = p;
 	SDL_mutexV(mpeg_mutex);
 }
@@ -919,7 +1027,6 @@ static void smpeg_pause(short p)
 static void smpeg_seek(gint time)
 {
 	SMPEG_Info info;
-
 	SDL_mutexP(mpeg_mutex);
 	SMPEG_getinfo(mpeg, &info);
 
@@ -931,10 +1038,18 @@ static void smpeg_seek(gint time)
 			SDL_mutexP(vcd_mutex);
 			vcdpos = msf_to_lba(0, (int) time, 0);
 			SDL_mutexV(vcd_mutex);
+			if(XMMSAUDIO)
+			{
+				smpeg_ip.output->flush((gint)time * 1000);
+			}
 		}
 		else
-#endif 
-			SMPEG_seek(mpeg, (gint) ((gdouble) time / (info.total_time) * info.total_size));
+#endif 		
+			SMPEG_seek(mpeg, (gint) ((gdouble)time / (info.total_time) * info.total_size));
+			if(XMMSAUDIO)
+			{
+				smpeg_ip.output->flush((gint)time*1000);
+			}
 	}
 	SDL_mutexV(mpeg_mutex);
 }
@@ -966,9 +1081,13 @@ static gint smpeg_get_time(void)
 	{
 		SMPEG_getinfo(mpeg, &info);
 		SDL_mutexV(mpeg_mutex);
-		
-		return (info.current_time * 1000);
-/*		return ((gint)info.current_offset / ((gint)info.total_size / (gint)info.total_time) * 1000); */
+		if(XMMSAUDIO)
+			return smpeg_ip.output->output_time();
+		else
+			return (info.current_time * 1000);
+/*
+		return (((info.current_offset / (info.total_size / info.total_time)) * 1000) - (smpeg_ip.output->written_time() - smpeg_ip.output->output_time()));
+*/
 	}
 }
 
@@ -1024,6 +1143,9 @@ void smpeg_read_config(void)
 	smpeg_cfg.wratio_mode = FALSE;
 	smpeg_cfg.window_mode = FALSE;
 	smpeg_cfg.filter_mode = FALSE;
+	smpeg_cfg.xmmsaudio_mode = FALSE;
+	smpeg_cfg.audioonly_mode = FALSE;
+	smpeg_cfg.samevideo_mode = FALSE;
 	smpeg_cfg.sizew_mode = 320;
 	smpeg_cfg.sizeh_mode = 200;
 
@@ -1047,6 +1169,12 @@ void smpeg_read_config(void)
 				&smpeg_cfg.window_mode);
 	xmms_cfg_read_boolean(cfg, "smpeg-xmms", "smpeg_filter",
 				&smpeg_cfg.filter_mode);
+	xmms_cfg_read_boolean(cfg, "smpeg-xmms", "smpeg_xmmsaudio",
+				&smpeg_cfg.xmmsaudio_mode);
+	xmms_cfg_read_boolean(cfg, "smpeg-xmms", "smpeg_audioonly",
+				&smpeg_cfg.audioonly_mode);
+	xmms_cfg_read_boolean(cfg, "smpeg-xmms", "smpeg_samevideo",
+				&smpeg_cfg.samevideo_mode);
 	xmms_cfg_read_int(cfg, "smpeg-xmms", "smpeg_sizew",
 				&smpeg_cfg.sizew_mode);
 	xmms_cfg_read_int(cfg, "smpeg-xmms", "smpeg_sizeh",
