@@ -1,6 +1,6 @@
 /*
  *  smpeg-xmms - (S)MPEG Plugin for XMMS
- *  Copyright (C) 2000 4Front Technologies
+ *  Copyright (C) 2000-2001 4Front Technologies
  *  http://www.opensound.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -16,7 +16,7 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
+ */ 
 
 #ifndef SMPEG_XMMS_H
 
@@ -34,6 +34,9 @@ typedef struct
 	gboolean wratio_mode;
 	gboolean window_mode;
 	gboolean filter_mode;
+	gboolean xmmsaudio_mode;
+	gboolean audioonly_mode;
+	gboolean samevideo_mode;
 	gint sizew_mode, sizeh_mode;
 }
 SmpegConfig;
